#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
@author: Olegs Nikisins
"""
#==============================================================================
# Import here:

from bob.pad.face.database import BatlPadDatabase

from torch import nn


#==============================================================================
# Define parameters here:

"""
Note: do not change names of the below constants.
"""
NUM_EPOCHS = 100  # Maximum number of epochs
BATCH_SIZE =  64  # Size of the batch
LEARNING_RATE = 1e-4  # Learning rate
NUM_WORKERS = 8  # The number of workers for the DataLoader


"""
Set the kwargs of the "dataset" instance of the DataFolder class.
Note: do not change the name ``kwargs``.
"""
# Initialize HLDI for data folder:
ORIGINAL_DIRECTORY = "" # this arguments is not important in this case
ORIGINAL_EXTENSION = ".h5"  # extension of the data files
ANNOTATIONS_TEMP_DIR = "" # this argument is not used here
PROTOCOL = 'grandtest-color*infrared*depth-10' # 3 channels are used here, 10 frames

bob_hldi_instance = BatlPadDatabase(protocol=PROTOCOL,
                                    original_directory=ORIGINAL_DIRECTORY,
                                    original_extension=ORIGINAL_EXTENSION,
                                    annotations_temp_dir=ANNOTATIONS_TEMP_DIR,  # annotations computed here will not be saved because ANNOTATIONS_TEMP_DIR is empty string
                                    landmark_detect_method="mtcnn",  # detect annotations using mtcnn
                                    exclude_attacks_list=['makeup'],
                                    exclude_pai_all_sets=True,  # exclude makeup from all the sets, which is the default behavior for grandtest protocol
                                    append_color_face_roi_annot=False)  # annotations defining ROI in the cropped face image are not important here
kwargs = {}
kwargs["data_folder"] = "NO NEED TO SET HERE, WILL BE SET IN THE TRAINING SCRIPT"
# NOTE: ``kwargs["transform"] = transform`` is re-defined below, after ``transform()`` method is defined
kwargs["transform"] = None # keep None for now, re-define below
kwargs["extension"] = '.hdf5'
kwargs["bob_hldi_instance"] = bob_hldi_instance
kwargs["hldi_type"] = "pad"
kwargs["groups"] = ['train']
kwargs["protocol"] = 'grandtest'
kwargs["purposes"] = ['real', 'attack']
kwargs["allow_missing_files"] = True


"""
Transformations to be applied to the input data sample.
Note: the variable or function name ``transform`` must be the same in
all configuration files. This transformation is handled in DataFolder.
"""
from bob.learn.pytorch.utils import MeanStdNormalizer

transform = MeanStdNormalizer(kwargs)

"""
Set the kwargs of the "dataset" instance of the DataFolder class.
Note: do not change the name ``kwargs``.
"""
# NOTE: re-define ``transform`` parameter, after we defined the ``transform`` method / object
# In this case transformation is mean-std normalization, given mean-std for each feature:
kwargs["transform"] = transform


"""
Define the network to be trained as a class, named ``Network``.
Note: Do not change the name of the below class.
"""
from bob.learn.pytorch.architectures import TwoLayerMLP as Network


"""
Define the kwargs to be used for ``Network`` in the dictionary namely
``network_kwargs``.
"""
network_kwargs = {}
network_kwargs['in_features'] = 1296
network_kwargs['n_hidden_relu'] = 10
network_kwargs['apply_sigmoid'] = True


"""
Define the loss to be used for training.
Note: do not change the name of the below variable.
"""
loss_type = nn.BCELoss()


"""
OPTIONAL: if not defined **above** loss will be computed in the training script.
See training script for details

Define the function to compute the loss. Don't change the signature of this
function: ``loss_function(output, input, target)``
"""

from bob.learn.pytorch.utils import weighted_bce_loss as loss_function

